#include "PointLight.hxx"

PointLight::PointLight(Scene* scene, const Vec3f& pos, const Vec3f& intensity)
  : Light(scene), 
    m_pos(pos),
    m_intensity(intensity)
{
}

PointLight::~PointLight()
{
}

PointLight::PointLight()
  : Light(0), 
    m_pos(Vec3f()),
    m_intensity(Vec3f())
{
}

bool
PointLight::Illuminate(Ray& shadow_ray, Vec3f& intensity)
{
  // distance to light source
  Vec3f dir = m_pos-shadow_ray.origin();
  float r = dir.norm()-Epsilon;
  float falloff = 1.0f/(r*r);
  
  
  intensity = m_intensity * falloff;
  
  // modify ray for shadow computation
  shadow_ray.setHit(0);
  // for shadow calculation
  shadow_ray.setT(r);
  // set direction to light source
  dir.normalize();
  shadow_ray.setDir(dir);
  
  return true;
}

const Vec3f&
PointLight::position() const
{
  return m_pos;
}

const Vec3f&
PointLight::intensity() const
{
  return m_intensity;
}

bool
PointLight::IsArea() 
{
  return false;
}
