#ifndef KDTREE_HXX
#define KDTREE_HXX 

#include <vector>

#include "Box.hxx"
#include "Primitive.hxx"

//! Interface for tree nodes
class kDTreeNode
{
public:
  virtual ~kDTreeNode(){;};
  // interface for the traversal. [t0,t1] code the active ray interval.
  virtual bool Traverse(Ray &ray, float& t0, float& t1) = 0;
};

//! An inner node of the kD-tree
class kDTreeInnerNode : public kDTreeNode
{
public:
  kDTreeInnerNode(float split, int axis);   
  virtual ~kDTreeInnerNode();
  
  virtual bool Traverse(Ray& ray, float& t0, float& t1);

  void setChildren(kDTreeNode* left, kDTreeNode* right);
private:
  kDTreeInnerNode();
  kDTreeInnerNode(const kDTreeInnerNode& );
  kDTreeInnerNode& operator=(const kDTreeInnerNode& );

  // pointer to the children
  kDTreeNode* m_children[2]; 
  // encodes the position of the splitting plane on the splitting axis
  float m_split;
  // encodes the splitting axis
  int m_axis;
};

//! A leaf node of the kD-tree
class kDTreeLeafNode : public kDTreeNode
{
public:
  kDTreeLeafNode(const std::vector<Primitive*>& prim);
  virtual ~kDTreeLeafNode();

  virtual bool Traverse(Ray& ray, float& t0, float& t1);
  
private:
  kDTreeLeafNode();
  kDTreeLeafNode(const kDTreeLeafNode& );
  kDTreeLeafNode& operator=(const kDTreeLeafNode& );

  std::vector<Primitive *> m_primitives;
};
  
//! The actual kDTree interface
class kDTree
{
public:
  kDTree(const Box& bounds, const std::vector<Primitive*>& prim);
  ~kDTree();
  
  kDTreeNode* BuildTree(const Box& bounds,
			const std::vector<Primitive *>& prim,
			int depth); 
  
  bool Intersect(Ray &ray); 
  
private:
  kDTree();
  kDTree(const kDTree& );
  kDTree& operator=(const kDTree& );

  // root node
  kDTreeNode* m_root;
  // bounding box of the entire tree
  Box m_bounds;
  // maximal recursion depth
  int m_maxDepth;
  // minimal number of triangles per node
  int m_minTri;
};
#endif
